@@ -1,6 +1,6 @@
 ;;; browse-url.el --- pass a URL to a WWW browser
 
-;; Copyright (C) 1995, 96, 97, 98, 99, 2000, 2001
+;; Copyright (C) 1995, 96, 97, 98, 99, 2000, 2001, 2002
 ;;   Free Software Foundation, Inc.
 
 ;; Author: Denis Howe <dbh@doc.ic.ac.uk>
@@ -44,6 +44,8 @@
 ;; browse-url-cci                     XMosaic     2.5
 ;; browse-url-w3                      w3          0
 ;; browse-url-w3-gnudoit              w3 remotely
+;; browse-url-w3m                     w3m         0.2.1, probably
+;; browse-url-w3m-gnudoit             w3m remotely 0.2.1, probably
 ;; browse-url-iximosaic               IXI Mosaic  ?
 ;; browse-url-lynx-*	              Lynx	     0
 ;; browse-url-grail                   Grail       0.3b1
@@ -540,6 +542,8 @@
 ;;      autoloading, use gnuclient instead of gnudoit.
 ;;1999-12-17 Neal Becker <nbecker@fred.net>
 ;;      * Add kfm support
+;;2002-11-16 Nix <nix@esperi.demon.co.uk>
+;;      * Add w3m support
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Code:
@@ -575,6 +579,9 @@ regexp should probably be \".\" to specify a default browser."
 	  (function-item :tag "Emacs W3" :value  browse-url-w3)
 	  (function-item :tag "W3 in another Emacs via `gnudoit'"
 			 :value  browse-url-w3-gnudoit)
+	  (function-item :tag "Emacs W3M" :value  browse-url-w3)
+	  (function-item :tag "W3M in another Emacs via `gnudoit'"
+			 :value  browse-url-w3-gnudoit)
 	  (function-item :tag "Mozilla" :value  browse-url-mozilla)
 	  (function-item :tag "Galeon" :value  browse-url-galeon)
 	  (function-item :tag "Netscape" :value  browse-url-netscape)
@@ -636,6 +643,7 @@ Defaults to the value of `browse-url-netscape-arguments' at the time
   :type '(choice string (const :tag "Default" nil))
   :group 'browse-url)
 
+;;;###autoload
 (defcustom browse-url-mozilla-program "mozilla"
   "*The name by which to invoke Mozilla."
   :type 'string
@@ -671,6 +679,13 @@ Defaults to the value of `browse-url-galeon-arguments' at the time
   :type '(repeat (string :tag "Argument"))
   :group 'browse-url)
 
+(defcustom browse-url-mozilla-new-window-is-tab nil
+  "*Whether to open up new windows in a tab or a new window.
+If non-nil, then open the URL in a new tab rather than a new window if
+`browse-url-mozilla' is asked to open it in a new window."
+  :type 'boolean
+  :group 'browse-url)
+
 (defcustom browse-url-galeon-new-window-is-tab nil
   "*Whether to open up new windows in a tab or a new window.
 If non-nil, then open the URL in a new tab rather than a new window if
@@ -804,7 +819,8 @@ highlighting, assuming the normal default for showing the cursor."
   :group 'browse-url)
 
 (defcustom browse-url-gnudoit-program "gnuclient" ;; XEmacs
-  "*The name of the `gnudoit' program used by `browse-url-w3-gnudoit'."
+  "*The name of the `gnudoit' program.
+Used by `browse-url-w3-gnudoit' and `browse-url-w3m-gnudoit'."
   :type 'string
   :group 'browse-url)
 
@@ -1112,8 +1128,9 @@ the effect of `browse-url-new-window-flag'.
 When called non-interactively, optional second argument NEW-WINDOW is
 used instead of `browse-url-new-window-flag'.
 
-The order attempted is gnome-moz-remote, Mozilla, Galeon, Konqueror,
-Netscape, Opera, Mosaic, IXI Mosaic, Lynx in an xterm, MMM, and then W3."
+The order attempted is emacs-w3m, gnome-moz-remote, Mozilla, Galeon,
+Konqueror,Netscape, Opera, Mosaic, IXI Mosaic, Lynx in an xterm, MMM,
+and then W3."
   (apply
     (cond
      ((executable-find "gnome-moz-remote") 'browse-url-gnome-moz)
@@ -1124,6 +1141,7 @@ Netscape, Opera, Mosaic, IXI Mosaic, Lynx in an xterm, MMM, and then W3."
      ((executable-find browse-url-opera-program) 'browse-url-opera)
      ((executable-find browse-url-mosaic-program) 'browse-url-mosaic)
      ((executable-find "tellw3b") 'browse-url-iximosaic)
+     ((fboundp 'w3m-goto-url-new-session) 'browse-url-w3m)
      ((executable-find browse-url-xterm-program) 'browse-url-lynx-xterm)
      ((executable-find "mmm") 'browse-url-mmm)
      (t 'browse-url-w3))
@@ -1207,8 +1225,19 @@ non-nil, load the document in a new Mozilla window, otherwise use a
 random existing one.  A non-nil interactive prefix argument reverses
 the effect of `browse-url-new-window-flag'.
 
+If `browse-url-mozilla-new-window-is-tab' is non-nil, then whenever a
+document would otherwise be loaded in a new window, it is loaded in a
+new tab in an existing window instead.
+
 When called non-interactively, optional second argument NEW-WINDOW is
-used instead of `browse-url-new-window-flag'."
+used instead of `browse-url-new-window-flag'.
+
+On MS-Windows systems the optional `new-window' parameter is ignored.
+Mozilla for Windows does not support the \"-remote\" command line
+parameter. Therefore the `browse-url-new-window-flag', 
+`browse-url-new-window-flag' and `browse-url-mozilla-new-window-is-tab'
+are ignored as well. Mozilla on Windows will always open the
+requested URL in a new window."
   (interactive (browse-url-interactive-arg "URL: "))
   ;; URL encode any `confusing' characters in the URL.  This needs to
   ;; include at least commas; presumably also close parens.
@@ -1216,16 +1245,24 @@ used instead of `browse-url-new-window-flag'."
     (setq url (replace-match
 	       (format "%%%x" (string-to-char (match-string 0 url))) t t url)))
   (let* ((process-environment (browse-url-process-environment))
-         (process (apply 'start-process
-			 (concat "mozilla " url) nil
-			 browse-url-mozilla-program
-			 (append
-			  browse-url-mozilla-arguments
-			  (list "-remote"
-				(concat "openURL("
-					url
-					(if new-window ",new-window")
-					")"))))))
+         (process
+	  (apply 'start-process
+		 (concat "mozilla " url) nil
+		 browse-url-mozilla-program
+		 (append
+		  browse-url-mozilla-arguments
+          (if (or (featurep 'dos-w32)
+                  (string-match "win32" system-configuration))
+              (list url)
+            (list "-remote"
+                  (concat "openURL("
+                          url
+                          (if (browse-url-maybe-new-window
+                               new-window)
+                              (if browse-url-mozilla-new-window-is-tab
+                                  ",new-tab")
+                            ",new-window")
+                          ")")))))))
     (set-process-sentinel process
 			  `(lambda (process change)
 			     (browse-url-mozilla-sentinel process ,url)))))
@@ -1467,6 +1504,38 @@ The `browse-url-gnudoit-program' program is used with options given by
 		   (list (concat "(w3-fetch \"" url "\")")
 			 "(raise-frame)"))))
 
+;; --- emacs-w3m ---
+
+;;;###autoload
+(defun browse-url-w3m (url &optional new-window)
+  "Ask the emacs-w3m WWW browser to load URL.
+Default to the URL around or before point.
+
+When called interactively, if variable `browse-url-new-window-flag' is
+non-nil, load the document in a new window.  A non-nil interactive
+prefix argument reverses the effect of `browse-url-new-window-flag'.
+
+When called non-interactively, optional second argument NEW-WINDOW is
+used instead of `browse-url-new-window-flag'."
+  (interactive (browse-url-interactive-arg "W3M URL: "))
+  (if (browse-url-maybe-new-window new-window)
+      (progn
+        (split-window) (other-window 1)))
+  (w3m-goto-url-new-session url))
+
+;;;###autoload
+(defun browse-url-w3m-gnudoit (url &optional new-window)
+  ;; new-window ignored
+  "Ask the Emacs running gnuserv to load the URL using the emacs-w3m browser.
+The `browse-url-gnudoit-program' program is used with options given by
+`browse-url-gnudoit-args'.  Default to the URL around or before point."
+  (interactive (browse-url-interactive-arg "W3M URL: "))
+    (apply 'start-process (concat "gnudoit:" url) nil
+	   browse-url-gnudoit-program
+	   (append browse-url-gnudoit-args
+		   (list (concat "(w3m-goto-url-new-session \"" url "\")")
+			 "(raise-frame)"))))
+
 ;; --- Lynx in an xterm ---
 
 ;;;###autoload
